#include "platform_api.h"
#include <msp/core/systemerror.h>
#include <msp/io/handle_private.h>
#include <msp/strings/format.h>
#include "datagramsocket.h"
#include "sockaddr_private.h"
#include "socket_private.h"

namespace Msp {
namespace Net {

DatagramSocket::DatagramSocket(Family f, int p):
	ClientSocket(f, SOCK_DGRAM, p)
{
	set_socket_events(S_INPUT);
}

bool DatagramSocket::connect(const SockAddr &addr)
{
	SockAddr::SysAddr sa = addr.to_sys();
	check_sys_connect_error(::connect(priv->handle, reinterpret_cast<const sockaddr *>(&sa.addr), sa.size));

	delete peer_addr;
	peer_addr = addr.copy();

	delete local_addr;
	SockAddr::SysAddr lsa;
	getsockname(priv->handle, reinterpret_cast<sockaddr *>(&lsa.addr), &lsa.size);
	local_addr = SockAddr::new_from_sys(lsa);

	connected = true;

	return true;
}

unsigned DatagramSocket::sendto(const char *buf, unsigned size, const SockAddr &addr)
{
	if(size==0)
		return 0;

	SockAddr::SysAddr sa = addr.to_sys();
	return check_sys_error(::sendto(priv->handle, buf, size, 0, reinterpret_cast<const sockaddr *>(&sa.addr), sa.size), "sendto");
}

unsigned DatagramSocket::recvfrom(char *buf, unsigned size, SockAddr *&from_addr)
{
	if(size==0)
		return 0;

	SockAddr::SysAddr sa;
	unsigned ret = check_sys_error(::recvfrom(priv->handle, buf, size, 0, reinterpret_cast<sockaddr *>(&sa.addr), &sa.size), "recvfrom");
	from_addr = SockAddr::new_from_sys(sa);

	return ret;
}

} // namespace Net
} // namespace Msp
